package com.bytesw.bytebot.model;

import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Getter;
import lombok.Setter;
import org.springframework.format.annotation.DateTimeFormat;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Date;
import java.util.List;


@Entity
@Table(name = "itinerary_reservations")
@Getter
@Setter
public class ItineraryReservation implements Serializable {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "reservation_id", nullable = false)
    private Long id;

    @Column(name = "date_reservation_made")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private Date dateReservation;

    @Column(name = "reservation_status_code")
    private String status;

    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    @JoinColumn(name = "passenger_id", referencedColumnName = "id")
    @JsonProperty(access = JsonProperty.Access.WRITE_ONLY)
    private Passenger passenger;

    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    @JoinColumn(name = "agent_id")
    @JsonProperty(access = JsonProperty.Access.WRITE_ONLY)
    private BookingAgent agent;

    @ManyToMany(fetch = FetchType.LAZY, cascade = { CascadeType.PERSIST, CascadeType.MERGE } )
    @JoinTable(name = "reservation_payments",
            joinColumns = { @JoinColumn(name = "reservation_id") },
            inverseJoinColumns = { @JoinColumn(name = "payment_id") })
    private List<Payment> payments;


    @ManyToMany(fetch = FetchType.LAZY, cascade = { CascadeType.PERSIST, CascadeType.MERGE, CascadeType.ALL})
    @JoinTable(name = "itinerary_legs",
            joinColumns = { @JoinColumn(name = "reservation_id") },
            inverseJoinColumns = { @JoinColumn(name = "leg_id") })
    private List<Leg> legs;

    @PrePersist
    public void prePersist() {
        dateReservation = new Date();
    }

    public void addPayment(Payment payment) {
        this.payments.add(payment);
        payment.getItineraryReservations().add(this);
    }

    public void addLeg(Leg leg) {
        this.legs.add(leg);
        leg.getItineraryReservations().add(this);
    }
}
