package com.bytesw.bytebot.model;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Getter;
import lombok.Setter;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

@Entity
@Table(name = "legs")
@Getter
@Setter
public class Leg implements Serializable {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    @Column(name = "actual_departure_time")
    @Temporal(TemporalType.TIMESTAMP)
    private Date departureDate;

    @Column(name = "actual_arrival_time")
    @Temporal(TemporalType.TIMESTAMP)
    private Date arrivalDate;

    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    @JoinColumn(name = "origin_airport_code")
    @JsonProperty(access = JsonProperty.Access.WRITE_ONLY)
    private Airport codeAirportOrigin;

    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    @JoinColumn(name = "destination_airport_code")
    @JsonProperty(access = JsonProperty.Access.WRITE_ONLY)
    private Airport codeAirportDestination;

    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    @JoinColumn(name = "flight_number")
    @JsonProperty(access = JsonProperty.Access.WRITE_ONLY)
    private FlightSchedule codeFlight;

    @ManyToMany(fetch = FetchType.LAZY, cascade = { CascadeType.PERSIST, CascadeType.MERGE, CascadeType.ALL  }, mappedBy="legs")
    @JsonIgnore
    private List<ItineraryReservation> itineraryReservations;
}
